{{
┌──────────────────────────────────────────────────────────────────────────────────────────────────────┐
│ Author: Dietmar Uhlig                                                                                │
│ Copyright (c) 2015 Dietmar Uhlig                                                                     │
│ See end of file for terms of use.                                                                    │
│ Die Nutzungsbedingungen befinden sich am Ende der Datei                                              │
└──────────────────────────────────────────────────────────────────────────────────────────────────────┘

Version:     0.01
Last Change: 2015-07-21

Log:
0.01, 2015-07-02: created

}}
' command line option parsing

#define __ERRORMESSAGES

OBJ
  ios: "reg-ios"

CON
  MAX_OPTS = 32
  PARA_MAXLEN = 64              ' as defined in reg-ios

  #0, X_NO_ERROR,
      X_INVAL_OPT_CHAR, X_DOUBLE_OPT_CHAR, X_OPT_SYNTAX,
      X_MISSING_VAL, X_UNKNOWN_OPT, X_HEAP_OVERFLOW

VAR
  byte num_opts            ' real number of possible options
  byte opt_char[MAX_OPTS]  ' allowed option chars, parsed by get_opts
  long opt_type            ' bit array for option type: 0 ... boolean, 1 ... string
  long opt_read            ' bit array of options read from command line
  byte next_para[PARA_MAXLEN + 1]
  byte this_opt            ' current option while parsing
  byte read_ahead          ' =1 if first parameter that is not an option was already read
  word opt_val_ptr[MAX_OPTS]      ' pointers to opt_val_heap
  word opt_val_free               ' pointer to first free byte in opt_val_heap
  byte opt_val_heap[PARA_MAXLEN]  ' string heap for option values
  byte last_err

'
' ----- Read Command Line ----------------------------------------------
'

PUB get_opts(opt_str) | i, x
' read actual command line options based on allowed options given by opt_str
  clear_opt_char
  parse_opt_str(opt_str)
  ios.parastart
  next_para[0]~ ' wirklich nötig?
  read_ahead~
  opt_val_free := @opt_val_heap
  read_cmd_line

PUB paranext(para_addr)
' handles the read-ahead of the first parameter after the options
  if read_ahead~
    bytemove(para_addr, @next_para, strsize(@next_para))
    return 1
  else
    return ios.paranext(para_addr)

PRI parse_opt_str(opt_str) | i, valid_opt_char
' parse the string of allowed options
  num_opts := 0
  valid_opt_char := 0
  repeat i from 0 to strsize(opt_str) - 1
    case byte[opt_str][i]
      ":":
        ifnot valid_opt_char
          die(X_OPT_SYNTAX)
        opt_type |= 1 << (num_opts - 1)
        valid_opt_char := 0
      "-":
        die(X_INVAL_OPT_CHAR)
      other:
        if opt2idx(byte[opt_str][i]) => 0
          die(X_DOUBLE_OPT_CHAR)
        opt_char[num_opts] := byte[opt_str][i]
        num_opts++
        valid_opt_char := 1

PRI read_cmd_line | i
  this_opt := 0
  repeat while ios.paranext(@next_para)
    i := opt2idx(this_opt)
    if next_para[0] == "-"
      if is_string_opt(i)  ' previous option required a value
        die(X_MISSING_VAL)
      if read_next_opt(@next_para + 1) == 0
        return
    else
      if is_string_opt(i)
        read_opt_val(i)
        this_opt := 0
      else
        ' a string that is not an option parameter marks the end of the options
        read_ahead := 1
        return
  if is_string_opt(opt2idx(this_opt))  ' previous option required a value...
    die(X_MISSING_VAL)                 ' ...but command line already ended

PRI read_next_opt(para_addr) | i
' check next command line parameter for option or value
  this_opt := byte[para_addr][0]
  if this_opt == "-"  ' -- marks the end of the options ' geht fälschlicherweise gut bei cmdline "-a-"
    return 0
  i := opt2idx(this_opt)
  if i => 0
    opt_read |= 1 << this_opt
  else
    die(X_UNKNOWN_OPT)

  if byte[para_addr][1] <> 0  ' means short notation of parameters, like "-ab" for "-a -b"
    if is_string_opt(i)
      die(X_MISSING_VAL)
    read_next_opt(para_addr + 1)
  return 1

PRI read_opt_val(idx) | s
  s := strsize(@next_para) + 1
  bytemove(opt_val_free, @next_para, s)
  opt_val_ptr[idx] := opt_val_free
  opt_val_free += s
  if opt_val_free > @opt_val_heap + PARA_MAXLEN
    die(X_HEAP_OVERFLOW)

'
' ----- Deliver Option Values ----------------------------------------------
'

PUB opt(option) : value | i
  i := opt2idx(option)
  if i < 0   ' you asked for an option that was not named in get_opts
    die(X_UNKNOWN_OPT)
  if is_string_opt(i)
    return opt_val_ptr[i]
  else
    return -((opt_read & (1 << i)) > 0)

'
' ----- Tools ----------------------------------------------
'

PRI die(exception)
  last_err := exception
  abort -exception

PRI opt2idx(o) | i
  repeat i from 0 to num_opts - 1
    if opt_char[i] == o
      return i
  return -1

PRI is_string_opt(opt_ix)
  if opt_ix < 0
    return 0
  return opt_type & (1 << opt_ix)

PRI clear_opt_char | i
  repeat i from 0 to MAX_OPTS - 1
    opt_char[i]~
    opt_val_ptr[i]~
  opt_type~

'
' ----- Error Messages ----------------------------------------------
'
#ifdef __ERRORMESSAGES

PUB last_errmsg
  return errmsg(last_err)

PUB errmsg(errcode) | e
  return X_ind2mem(errcode)

PRI X_ind2mem(index)
  return lookup(index - X_NO_ERROR:                                   {{
  }}            @M_INVAL_OPT_CHAR, @M_DOUBLE_OPT_CHAR, @M_OPT_SYNTAX, {{
  }}            @M_MISSING_VAL, @M_UNKNOWN_OPT, @M_HEAP_OVERFLOW)

DAT

M_INVAL_OPT_CHAR        byte    "invalid option char", 0
M_DOUBLE_OPT_CHAR       byte    "double option char", 0
M_OPT_SYNTAX            byte    "option syntax error", 0
M_MISSING_VAL           byte    "missing value", 0
M_UNKNOWN_OPT           byte    "unknown option", 0
M_HEAP_OVERFLOW         byte    "option heap overflow", 0

#endif

DAT

{{

┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}

