{{
┌──────────────────────────────────────────────────────────────────────────────────────────────────────┐
│ Author: Dietmar Uhlig                                                                                │
│ Copyright (c) 2015 Dietmar Uhlig                                                                     │
│ See end of file for terms of use.                                                                    │
│ Die Nutzungsbedingungen befinden sich am Ende der Datei                                              │
└──────────────────────────────────────────────────────────────────────────────────────────────────────┘

Version:     0.01
Last Change: 2015-07-21

Log:
0.01, 2015-07-08: created

}}


'test cases for simple options
CON

_CLKMODE     = XTAL1 + PLL16X
_XINFREQ     = 5_000_000


OBJ
  so: "simple_opts"
  ios: "reg-ios"

VAR
  byte verbose
  byte para_buf[so#PARA_MAXLEN] ' PARA_MAXLEN + 1???

PUB main | exception, k
  ios.start
  ios.printnl
  ios.print(string("Verbose output? [y]/n"))
  repeat
    k := ios.keywait
    if k == "n" or k == "N"
      verbose := 0
      ios.printnl
      quit
    if k == "y" or k == "Y" or k == ios#CHAR_NL
      verbose := 1
      ios.printcls
      quit
  exception := \run_all_testcases
  if exception <> 0
    ios.print(string(" ABORT "))
    ios.printdec(exception)
    ios.printnl
  ios.stop

PRI run_all_testcases | t, i, k
  repeat i from 1 to NUM_TESTCASES
    run_testcase(i, @tc1 + 4 * TC_LEN * (i - 1))
    if verbose
      ios.printnl
      ios.print(string("Continue? [y]/n"))
      k := ios.keywait
      if k == "n" or k == "N"
        ios.printnl
        quit
      else
        ios.printcls
  ios.printnl
  pr_ss(string("Bye!"), 0, 1)

PRI run_testcase(num_tc, tc) | x
  print_tc(num_tc, tc)

  ios.paraset(ind2mem(long[tc][CMDLINE_IDX]))
  x := \try(tc)
  chk_errcode(tc, x)

PRI try(tc) | val, o
  so.get_opts(ind2mem(long[tc][OPTS_IDX]))
  if verbose
    ios.print(string("Result"))
    ios.printnl
    ios.printnl
  o := long[tc][OPT_CHK]
  if o == 0                          ' then check remaining parameters
    repeat long[tc][EXP_TYPNVAL]
      if so.paranext(@para_buf) == 0 ' 0 means: parameter not found
        para_buf[0] := 0
        quit
    chk_str(ind2mem(long[tc][EXP_STR_IDX]), @para_buf, "p", long[tc][EXP_TYPNVAL])
  else                               ' check option value
    val := so.opt(o)
    if long[tc][EXP_TYPNVAL] == -1   ' shall be a string
      chk_str(ind2mem(long[tc][EXP_STR_IDX]), val, "o", o)
    else                             ' shall be a boolean value
      chk_bool(long[tc][EXP_TYPNVAL], val, o)

PRI chk_str(exp, got, opt_or_par, opt)
  if verbose
    if opt_or_par == "o"
      pr_sc(string(" text option "), opt, 0)
    else
      pr_sd(string(" parameter #"), opt, 0)
    ios.print(string(": got '"))
    pr_substr(got, so#PARA_MAXLEN)
  if strcomp(exp, got)
    if verbose
      ios.print(string("' as expected ('"))
      pr_substr(exp, so#PARA_MAXLEN)
      ios.print(string("')"))
      ios.printnl
  else
    if verbose
      ios.print(string("', expected '"))
      pr_substr(exp, so#PARA_MAXLEN)
      ios.print(string("'"))
      ios.printnl
    abort 1

PRI chk_bool(exp, got, opt)
  if verbose
    pr_sc(string(" bool option "), opt, 0)
    pr_sd(string(": got "), got, 0)
  if exp == got
    if verbose
      ios.print(string(" as expected"))
      ios.printnl
  else
    if verbose
      pr_sd(string(", expected "), exp, 1)
    abort 1

PRI chk_errcode(tc, x)
  if long[tc][EXP_ERR] == x
    pr_ok
  else
    pr_fail(x)

PRI pr_ok
  ios.setcolor(1)
  if verbose
    ios.print(string(" ...passed"))
    ios.printnl
  else
    ios.printchar(".")
  ios.setcolor(0)

PRI pr_fail(x)
  if verbose
    if x =< 0
      pr_sd(string("error code "), x, 0)
      pr_ss(string("("), so.last_errmsg, 0)
      ios.printchar(")")
    ios.setcolor(2)
    ios.print(string(" ...failed"))
    ios.printnl
  else
    ios.setcolor(2)
    ios.printchar("F")
  ios.setcolor(0)

PRI print_tc(num_tc, tc)
  if verbose
    pr_sd(string("Test Case #"), num_tc, 1)
    ios.printnl
    pr_ss(string(" opt string: "), ind2mem(long[tc][OPTS_IDX]), 1)
    pr_ss(string(" cmd line:   "), ind2mem(long[tc][CMDLINE_IDX]), 1)
    if long[tc][EXP_ERR] <> 0
      if long[tc][OPT_CHK] <> 0  ' check invalid option request
        print_exp_opt(tc)
      pr_sd(string(" exp. error: "), long[tc][EXP_ERR], 0)
      pr_ss(string(" - "), so.errmsg(-long[tc][EXP_ERR]), 1)
    else
      if long[tc][OPT_CHK] == 0  ' check parameter
        pr_sd(string(" exp. par. #"), long[tc][EXP_TYPNVAL], 0)
        pr_ss(string(": "), ind2mem(long[tc][EXP_STR_IDX]), 1)
      else
        print_exp_opt(tc)
    ios.printnl

PRI print_exp_opt(tc)
  if long[tc][EXP_TYPNVAL] == -1  ' check text option
    pr_sc(string(" check text opt. '"), long[tc][OPT_CHK], 0)
    pr_ss(string("': "), ind2mem(long[tc][EXP_STR_IDX]), 1)
  else
    pr_sc(string(" check bool opt. '"), long[tc][OPT_CHK], 0)
    pr_sd(string("': "), long[tc][EXP_TYPNVAL], 1)

PRI pr_substr(str, len) | c
  if strsize(str) =< len
    ios.print(str)
  else
    c := byte[str][len]
    byte[str][len] := 0
    ios.print(str)
    byte[str][len] := c
    ios.print(string("..."))

PRI pr_ss(s1, s2, nl)
  ios.print(s1)
  ios.print(s2)
  if nl
    ios.printnl

PRI pr_sd(s1, d1, nl)
  ios.print(s1)
  ios.printdec(d1)
  if nl
    ios.printnl

PRI pr_sc(s1, c1, nl)
  ios.print(s1)
  ios.printchar(c1)
  if nl
    ios.printnl

OBJ
' -----------------------------------------------------------
' Test data
'
CON
  #1, i_tc_opt1,  i_tc_opt2,  i_tc_opt3,  i_tc_cmdl1, i_tc_cmdl2,
      i_tc_cmdl3, i_tc_cmdl4, i_tc_cmdl5, i_tc_cmdl6, i_tc_opt4,
      i_tc_opt5,  i_tc_cmdl7, i_tc_exp1,  i_tc_opt6,  i_tc_cmdl8,
      i_tc_exp2,  i_tc_exp3,  i_tc_exp4,  i_tc_exp5,  i_tc_cmdl9

PRI ind2mem(index)
  return lookup(index: {{
  }}            @tc_opt1,  @tc_opt2,  @tc_opt3,  @tc_cmdl1, @tc_cmdl2, {{
  }}            @tc_cmdl3, @tc_cmdl4, @tc_cmdl5, @tc_cmdl6, @tc_opt4,  {{
  }}            @tc_opt5,  @tc_cmdl7, @tc_exp1,  @tc_opt6,  @tc_cmdl8, {{
  }}            @tc_exp2,  @tc_exp3,  @tc_exp4,  @tc_exp5,  @tc_cmdl9)

CON
' Test case data structure position names (to simulate a record access via named array index)
  #0, OPTS_IDX,     ' index to string of allowed options, parameter to get_opts
      CMDLINE_IDX,  ' command line to be simulated
      EXP_ERR,      ' expected error code (0 if no error)
      OPT_CHK,      ' option to check (if no error), empty means check of remaining parameters
      EXP_TYPNVAL,  ' option type and value: 0/1 = boolean false/true, -1 = string, or number of parameter to check
      EXP_STR_IDX,  ' index to expected option value, if type is string, or to expected parameter
      TC_LEN        ' NO Position but test case structure length

' possible vector format:
' 1. expect an error:
'    [idx], [idx], [errcode], 0, 0, 0
' 2. expect a boolean option:
'    [idx], [idx], 0, [opt], [0/1], 0
' 3. expect a string option:
'    [idx], [idx], 0, [opt], -1, [idx]
' 4. check the nth parameter after the options:
'    [idx], [idx], 0, 0, [n], [idx]
'
DAT

NUM_TESTCASES long      18

tc1           long      i_tc_opt1, i_tc_cmdl1, -so#X_INVAL_OPT_CHAR,  0, 0, 0
tc2           long      i_tc_opt2, i_tc_cmdl1, -so#X_DOUBLE_OPT_CHAR, 0, 0, 0
tc3           long      i_tc_opt3, i_tc_cmdl2, -so#X_MISSING_VAL,     0, 0, 0
tc4           long      i_tc_opt3, i_tc_cmdl3, -so#X_MISSING_VAL,     0, 0, 0
tc5           long      i_tc_opt3, i_tc_cmdl4, -so#X_MISSING_VAL,     0, 0, 0
tc6           long      i_tc_opt3, i_tc_cmdl5, -so#X_UNKNOWN_OPT,     0, 0, 0
tc7           long      i_tc_opt3, i_tc_cmdl6, -so#X_UNKNOWN_OPT,   "c", 0, 0
tc8           long      i_tc_opt4, i_tc_cmdl1, -so#X_OPT_SYNTAX,      0, 0, 0
tc9           long      i_tc_opt5, i_tc_cmdl1, -so#X_OPT_SYNTAX,      0, 0, 0
tc10          long      i_tc_opt3, i_tc_cmdl7, 0,   0,  1, i_tc_exp1
tc11          long      i_tc_opt6, i_tc_cmdl8, 0, "2",  1, 0
tc12          long      i_tc_opt6, i_tc_cmdl8, 0, "4",  0, 0
tc13          long      i_tc_opt6, i_tc_cmdl8, 0, "1", -1, i_tc_exp2
tc14          long      i_tc_opt6, i_tc_cmdl8, 0, "3", -1, i_tc_exp3
tc15          long      i_tc_opt6, i_tc_cmdl8, 0,   0,  1, i_tc_exp4
tc16          long      i_tc_opt6, i_tc_cmdl8, 0,   0,  2, i_tc_exp5
tc17          long      i_tc_opt6, i_tc_cmdl8, 0,   0,  3, i_tc_cmdl1
tc18          long      i_tc_opt6, i_tc_cmdl9, 0,   0,  1, i_tc_cmdl9

tc_opt1       byte      "-", 0
tc_opt2       byte      "aba", 0
tc_opt3       byte      "ab:", 0
tc_opt4       byte      ":", 0
tc_opt5       byte      "a::", 0
tc_opt6       byte      "1:23:4", 0

tc_cmdl1      byte      0
tc_cmdl2      byte      "-ab", 0
tc_cmdl3      byte      "-ba", 0
tc_cmdl4      byte      "-b -a", 0
tc_cmdl5      byte      "-c", 0
tc_cmdl6      byte      "-a", 0
tc_cmdl7      byte      "-a -- -c", 0
tc_cmdl8      byte      "-1 Erstens -23 kommt es anders...", 0
tc_cmdl9      byte      "123456789012345678901234567890123456789012345678901234567890123", 0

tc_exp1       byte      "-c", 0
tc_exp2       byte      "Erstens", 0
tc_exp3       byte      "kommt", 0
tc_exp4       byte      "es", 0
tc_exp5       byte      "anders...", 0

DAT

{{

┌──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
│                                                   TERMS OF USE: MIT License                                                  │
├──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
│Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    │
│files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    │
│modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software│
│is furnished to do so, subject to the following conditions:                                                                   │
│                                                                                                                              │
│The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.│
│                                                                                                                              │
│THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          │
│WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         │
│COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   │
│ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         │
└──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
}}

